local mobjscale = CV_RegisterVar({ 
	name = "kartgrow",					// Console variable to keep track
	defaultvalue = 0,					// for speed shoes replacement
	flags = CV_NETVAR | CV_SHOWMODIF,	// mechanic straight from SRB2Kart.
	PossibleValue = CV_YesNo
})

function A_TokenWindowsReward()
	for player in players.iterate
		P_GivePlayerRings(player, 50) // It's raining rings if Windows touches the token with all emeralds collected.
		S_StartSound(player.mo, sfx_cdfm67)
	end
end

// Redefining these functions to allow for stacking of the timers.
// For both when the Growth replication is on or off.
function A_SuperSneakers(actor)
	if actor.target.skin != "windows" then super(actor) return end
	if mobjscale.value == 1
		actor.target.player.powers[pw_sneakers] = $ + ((3*sneakertics) / 5) + 1
		if actor.target.player.powers[pw_super] == 0 then
			S_ChangeMusic("KGROW", true, player)
			S_StartMusicCaption("Growth Powerup", actor.target.player.powers[pw_sneakers], player)
		end
		S_StartSound(actor.target.player.mo, sfx_kc5a)
	else
		actor.target.player.powers[pw_sneakers] = $ + sneakertics + 1
		if actor.target.player.powers[pw_super] == 0 then
			P_PlayJingle(player, JT_SHOES)
			S_StartMusicCaption("Speed Shoes", actor.target.player.powers[pw_sneakers], player)
		end
	end
end

function A_Invincibility(actor)
	if actor.target.skin != "windows" then super(actor) return end
	actor.target.player.powers[pw_invulnerability] = $ + invulntics + 1
	if mobjscale.value == 1 and actor.target.player.powers[pw_sneakers] != 0 then return
	elseif actor.target.player.powers[pw_super] != 0 then return
	else
		P_PlayJingle(player, JT_INV);
		S_StartMusicCaption("Invincibility", actor.target.player.powers[pw_invulnerability], player)
	end
end

addHook("ThinkFrame", function()
	for player in players.iterate
		if (player.mo and player.mo.valid) and player.mo.skin == "windows" then
			if player.isxmomentum == nil or player.normalspeed < skins[player.mo.skin].normalspeed then
				player.normalspeed = skins[player.mo.skin].normalspeed
			end
			player.acceleration = skins[player.mo.skin].acceleration
			player.thrustfactor = skins[player.mo.skin].thrustfactor
			player.jumpfactor = skins[player.mo.skin].jumpfactor + (player.speed / 100)
			player.mo.destscale = FRACUNIT
			if player.speed > 20*FRACUNIT and player.powers[pw_super] == 0 then
				player.charflags = $ | SF_CANBUSTWALLS // Ram into the wall!
			else
				player.charflags = $ & ~SF_CANBUSTWALLS
			end
			if player.powers[pw_invulnerability] != 0 and player.powers[pw_sneakers] == 0 then
				// Kart mechanics begin...
				if player.isxmomentum == nil or player.normalspeed < (skins[player.mo.skin].normalspeed * 4) / 3 then
					player.normalspeed = (skins[player.mo.skin].normalspeed * 4) / 3
				end
				player.acceleration = (skins[player.mo.skin].acceleration * 4) / 3
			elseif player.powers[pw_sneakers] != 0 then
				// Before Kart's influence goes further...
				if mobjscale.value == 1 then
					player.mo.destscale = 2 * FRACUNIT // Grow!
					if player.powers[pw_invulnerability] < player.powers[pw_sneakers] then
						player.powers[pw_invulnerability] = player.powers[pw_sneakers]
					end
				else // Else, nothing exciting.
					player.thrustfactor = (skins[player.mo.skin].thrustfactor * 5) / 4
				end
			end
			if player.powers[pw_super] != 0 then
				player.charflags = $ | (SF_CANBUSTWALLS | SF_MULTIABILITY)
				player.powers[pw_invulnerability] = 0
				if mobjscale.value == 0 then
					player.powers[pw_sneakers] = 0
				end
				if player.hyper == nil or player.hyper.capable == false then
					if player.isxmomentum == nil or player.normalspeed < (skins[player.mo.skin].normalspeed * 3) / 2 then
						player.normalspeed = (skins[player.mo.skin].normalspeed * 3) / 2
					end
					player.acceleration = (skins[player.mo.skin].acceleration * 4) / 3
					player.thrustfactor = (skins[player.mo.skin].thrustfactor * 61) / 51
				elseif player.hyper.capable == true then
					// Extreme overclocking by XMomentum! Careful!
					if player.normalspeed < (skins[player.mo.skin].normalspeed * 7) / 4 then
						player.normalspeed = (skins[player.mo.skin].normalspeed * 7) / 4
					end
					player.acceleration = (skins[player.mo.skin].acceleration * 3) / 2
					player.thrustfactor = (skins[player.mo.skin].thrustfactor * 3) / 2
				end
			else
				player.charflags = $ & ~SF_MULTIABILITY // No extra jumps if not Super.
			end
			if player.powers[pw_underwater] < 30*TICRATE or player.powers[pw_spacetime] < 30*TICRATE then
				player.powers[pw_underwater] = 30*TICRATE // We already have machines underwater.
				player.powers[pw_spacetime] = 30*TICRATE // And it's cold in space.
			end
			if player.mo.destscale > player.mo.scale then
				if not (S_SoundPlaying(player.mo, sfx_kc5a)) then
					S_StartSound(player.mo, sfx_kc5a)
				end
			elseif player.mo.destscale < player.mo.scale then
				if not (S_SoundPlaying(player.mo, sfx_kc59)) then
					S_StartSound(player.mo, sfx_kc59)
				end
			end
			if player.exiting == (14*TICRATE)/5 then P_PlayVictorySound(player.mo, player) end
			// Transform at any point as long as you have rings and emeralds,
			// regardless of a shield being present. Use wisely.
			if player.cmd.buttons == BT_CUSTOM3 and player.powers[pw_super] == 0 then
				if player.rings >= 50 and emeralds >= 127 then
					P_DoSuperTransformation(player, false)
				end
			end
		end
	end
end)

// Token-to-emerald compilation/conversion.
// Will only work with Windows (the character, not your OS).
// The character is not planned for use in NiGHTS special stages,
// so this is implemented as an alternative method of getting emeralds.
addHook("TouchSpecial", function()
	for player in players.iterate
		if (player.mo and player.mo.valid) and player.mo.skin == "windows" then
			if token >= 0 then token = $ - 1 end
			if emeralds & EMERALD1 == 0 then
				emeralds = $ | EMERALD1
			elseif emeralds & EMERALD2 == 0 then
				emeralds = $ | EMERALD2
			elseif emeralds & EMERALD3 == 0 then
				emeralds = $ | EMERALD3
			elseif emeralds & EMERALD4 == 0 then
				emeralds = $ | EMERALD4
			elseif emeralds & EMERALD5 == 0 then
				emeralds = $ | EMERALD5
			elseif emeralds & EMERALD6 == 0 then
				emeralds = $ | EMERALD6
			elseif emeralds & EMERALD7 == 0 then
				emeralds = $ | EMERALD7
			else A_TokenWindowsReward() end
		end
	end
end, MT_TOKEN)

// Set up a value to check against for custom jingles.
local mplayer = CV_RegisterVar({
	name = "canyon.mid",
	defaultvalue = 1,
	flags = CV_SHOWMODIF,
	PossibleValue = CV_YesNo
})

// Script by Katmint.
// Edited w/ variable check.
// Redundant, but, just in case there's other sources of music change.

addHook("MusicChange", function(oldname, newname)
	if consoleplayer then
		local character = skins[consoleplayer.skin].name
		if character == "windows" then
			if oldname == "_1up" and consoleplayer.powers[pw_sneakers] != 0 and mobjscale.value == 1 then
				return "KGROW"
			elseif newname == "_super" and mplayer.value != 0 then
				return "ERRORMIX"
			elseif newname == "_inv" and mplayer.value != 0 then
				return "WINV"
			elseif newname == "_shoes" then
				if mobjscale.value == 1 then return "KGROW"
				elseif mplayer.value != 0 then return "WINFAST"
				end
			elseif newname == "HYPR" and mplayer.value != 0 then
				return "HYPWIN" // Coming through, XMomentum!
			end
		end
	end
end)